/*

	I used to hate the Freedoom lizard baby Commander Keen replacement
	and replaced it with a different spriteset for years.

	But it occurred to me that the only real reason for it is because
	the original sprite is for an inert corpse and the baby lizard just
	sits there motionless like some awful mockery of a living thing.

	This mod adds some sprites and fixes the animation by:
	- adding separate actors for the falling goop and lizard baby
	- adding a better splashing animation for the liquid goop
	- adding falling impact and blinking animations to the lizard baby
	- allowing the lizard baby to leave the container location

	2022-12-02
	verdantdregs@shaw.ca

*/

version "4.8"

class LizardJar:Actor replaces CommanderKeen{
	default{
		health 100;
		radius 16;
		height 68;
		deathheight 68;
		mass 10000000;
		painchance 256;
		+solid
		+spawnceiling
		+ceilinghugger
		+nogravity
		+nodamagethrust
		+dontfall
		+shootable
		+countkill
		+noicedeath
		+ismonster
		+noblood
		+nospriteshadow
		painsound "keen/pain";
		deathsound "keen/death";
	}
	override void postbeginplay(){
		super.postbeginplay();
		setz(ceilingz-height);
	}
	states{
	spawn:
		KEEN A -1;
		wait;
	pain:
		KEEN M 4 {if(health<50)bnoblood=false;}
		KEEN M 8 A_Pain();
		goto spawn;
	death:
		KEEN M 4;
		KEEN B 4 A_Scream();
		KEEN C 4 A_StartSound("misc/icebreak",CHAN_BODY,CHANF_OVERLAP,pitch:1.2);
		KEEN D 4 A_NoBlocking();
		KEEN E 4;
		KDRP A 40{
			let rrr=spawn("JarGoop",(pos.xy,pos.z-3),ALLOW_REPLACE);
			if(!rrr)return;
			rrr.vel.z=-getgravity()*2.;
		}
		KDRP B 100{
			let bb=spawn("JarLizard",(pos.xy,pos.z+(64-34)),ALLOW_REPLACE);
			if(!bb)return;
		if(!!target&&!!target.player)bb.friendplayer=target.playernumber()+1;
			bb.vel.z+=getgravity()*1.5;
		}
		KDRP B 0 A_KeenDie();
		KDRP B random(30,300) spawn("Blood",
			(pos.xy+(frandom(-5,5),frandom(-5,5)),pos.z+frandom(50,54)),
			ALLOW_REPLACE
		);
		wait;
	}
}
class JarLizard:Actor{
	default{
		+friendly
		-solid
		-shootable
		+noblockmap
		+castspriteshadow
		height 21;
		radius 9;

		speed 2;
		maxdropoffheight 1024;
		maxstepheight 3;
	}
	states{
	spawn:
	fall:
		LBBY A 0 nodelay A_JumpIf(floorz>=pos.z,"sit");
		LBBY AAAB 3 A_JumpIf(floorz>=pos.z,"splat");
		wait;
	splat:
		LBBY C 0 A_StartSound("lizard/splat");
		LBBY CDE 3;
	sit:
		LBBY E 1 A_SetTics(random(random(6,40),100));
		LBBY F random(2,3) A_Jump(42,"think","stretch","stretch","creep");
		loop;
	stretch:
		LBBY FD 4;
		LBBY C 1 A_SetTics(random(20,60));
		LBBY DF 6;
		goto sit;
	creep:
		LBBY FDCDE 4 A_Wander();
		LBBY F 0 A_Jump(128,"creep");
		goto sit;
	think:
		LBBY F 20;
		LBBY F 0{
			speed=17.;
			maxstepheight=1024;
			for(int i=0;i<MAXPLAYERS;i++){
				if(
					playeringame[i]
					&&!!players[i].mo
					&&!(
						abs(players[i].mo.pitch)<70
						&&absangle(players[i].mo.angle,players[i].mo.angleto(self))>150
					)
					&&players[i].mo.checksight(self)
				){
					spawn("TeleportFog",pos,ALLOW_REPLACE);
					return;
				}
			}
		}
	disappear:
		TNT1 AAAAAAAAA 10 A_Wander();
		TNT1 A 10{
			int fp=friendplayer-1;
			if(
				fp>=0
				&&fp<MAXPLAYERS
				&&playeringame[fp]
				&&!!players[fp].mo
				&&!random(0,7)
			){
				bnogravity=true;
				bfloat=true;
				setorigin(players[fp].mo.pos,false);
				return;
			}
			for(int i=0;i<MAXPLAYERS;i++){
				if(
					playeringame[i]
					&&!!players[i].mo
					&&!(
						abs(players[i].mo.pitch)<70
						&&absangle(players[i].mo.angle,players[i].mo.angleto(self))>150
					)
					&&players[i].mo.checksight(self)
				){
					return;
				}
			}
			bnogravity=false;
			bfloat=false;
			speed=default.speed;
			maxstepheight=default.maxstepheight;
			setstatelabel("fall");
		}
		loop;
	}
}
class JarGoop:Actor{
	default{
		-solid
		-shootable
		+noblockmap
		+castspriteshadow
		height 8;
		radius 16;
	}
	void A_SplashBlood(){
		int gbg;actor bld;
		[gbg,bld]=A_SpawnItemEx("Blood",
			frandom(0,radius),0,(frame==5?0:frandom(0,height-20)),
			frandom(0,1),0,frandom(2,4)*getgravity(),
			frandom(0,360),
			SXF_NOCHECKPOSITION
		);
		bld.A_StopAllSounds();  //stop mdt punch sound
	}
	states{
	spawn:
	fall:
		KSPL A 2 A_JumpIf(floorz>=pos.z,"splat");
		wait;
	splat:
		KSPL B 0{
			A_StartSound("misc/gibbed",pitch:1.8);
			bcastspriteshadow=false;
		}
		//KSPL BBCCDDEEFF 2 A_SplashBlood();
		KSPL BCDE 3;
		KSPL F -1;
		stop;
	}
}
